package java.util;

import java.util.function.DoubleConsumer;
import java.util.function.LongConsumer;
import java.util.function.Consumer;
import java.util.function.IntConsumer;

public interface PrimitiveIterator<T, T_CONS> extends Iterator<T>
{
    public abstract void forEachRemaining(T_CONS p0);
    
    public interface OfInt extends PrimitiveIterator<Integer, IntConsumer>
    {
        public abstract int nextInt();
        
        default public void forEachRemaining(IntConsumer intConsumer) {
            Objects.requireNonNull(intConsumer);
            while (this.hasNext()) {
                intConsumer.accept(this.nextInt());
            }
        }
        
        default public Integer next() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(this.getClass(), "{0} calling PrimitiveIterator.OfInt.nextInt()");
            }
            return Integer.valueOf(this.nextInt());
        }
        
        default public void forEachRemaining(Consumer<? super Integer> consumer) {
            if ((consumer instanceof IntConsumer)) {
                this.forEachRemaining((IntConsumer)consumer);
            }
            else {
                Objects.requireNonNull(consumer);
                if (Tripwire.ENABLED) {
                    Tripwire.trip(this.getClass(), "{0} calling PrimitiveIterator.OfInt.forEachRemainingInt(action::accept)");
                }
                this.forEachRemaining(consumer::accept);
            }
        }
    }
    
    public interface OfLong extends PrimitiveIterator<Long, LongConsumer>
    {
        public abstract long nextLong();
        
        default public void forEachRemaining(LongConsumer longConsumer) {
            Objects.requireNonNull(longConsumer);
            while (this.hasNext()) {
                longConsumer.accept(this.nextLong());
            }
        }
        
        default public Long next() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(this.getClass(), "{0} calling PrimitiveIterator.OfLong.nextLong()");
            }
            return Long.valueOf(this.nextLong());
        }
        
        default public void forEachRemaining(Consumer<? super Long> consumer) {
            if ((consumer instanceof LongConsumer)) {
                this.forEachRemaining((LongConsumer)consumer);
            }
            else {
                Objects.requireNonNull(consumer);
                if (Tripwire.ENABLED) {
                    Tripwire.trip(this.getClass(), "{0} calling PrimitiveIterator.OfLong.forEachRemainingLong(action::accept)");
                }
                this.forEachRemaining(consumer::accept);
            }
        }
    }
    
    public interface OfDouble extends PrimitiveIterator<Double, DoubleConsumer>
    {
        public abstract double nextDouble();
        
        default public void forEachRemaining(DoubleConsumer doubleConsumer) {
            Objects.requireNonNull(doubleConsumer);
            while (this.hasNext()) {
                doubleConsumer.accept(this.nextDouble());
            }
        }
        
        default public Double next() {
            if (Tripwire.ENABLED) {
                Tripwire.trip(this.getClass(), "{0} calling PrimitiveIterator.OfDouble.nextLong()");
            }
            return Double.valueOf(this.nextDouble());
        }
        
        default public void forEachRemaining(Consumer<? super Double> consumer) {
            if ((consumer instanceof DoubleConsumer)) {
                this.forEachRemaining((DoubleConsumer)consumer);
            }
            else {
                Objects.requireNonNull(consumer);
                if (Tripwire.ENABLED) {
                    Tripwire.trip(this.getClass(), "{0} calling PrimitiveIterator.OfDouble.forEachRemainingDouble(action::accept)");
                }
                this.forEachRemaining(consumer::accept);
            }
        }
    }
}
